#!/bin/bash
set -e
set -o pipefail

export MASTER_CONFIG_DIR="$CONFIG_DIR/master"

__update_api_ip() {
  log_proc_msg "Updating api service IP"
  local existing_service=$(sudo kubectl \
    --kubeconfig=$KUBECONFIG \
    get services -o json \
    | jq '.items[] | select(.metadata.name == "api")')
  if [ -z "$existing_service" ]; then
    log_proc_error "api service not created in k8s cluster, exiting"
    exit 1
  else
    log_proc_msg "Found api service, updating IP address"
    local api_ip=$(echo "$existing_service" \
      | jq -r '.spec.clusterIP')
    set_config "API_IP" "$api_ip"
  fi
}

__expose_api_endpoint() {
  if [ ! -f "$KUBECONFIG" ]; then
    log_proc_error "No kubeconfig file present, exiting"
    exit 1
  fi

  local existing_service=$(sudo kubectl \
    --kubeconfig=$KUBECONFIG \
    get services -o json \
    | jq '.items[] | select(.metadata.name == "api")')
  if [ -z "$existing_service" ]; then
    log_proc_msg "Exposing service endpoint for api"
    local service_config="$MASTER_CONFIG_DIR/api_service.yml"
    kubectl --kubeconfig $KUBECONFIG \
      apply -f "$service_config"
  else
    log_proc_msg "API service already exists, skipping"
  fi
}

__generate_api_service() {
  log_proc_msg "Generating service config for api"

  local api_service_template="$INSTALLER_SCRIPTS_DIR/configs/api_service.yml.template"
  local api_service="$MASTER_CONFIG_DIR/api_service.yml"

  cp -vr "$api_service_template" "$api_service"

  sed -i "s#{{COMPONENT}}#api#g" $api_service
  sed -i "s#{{API_PORT}}#$API_PORT#g" $api_service
}

__update_www_ip() {
  log_proc_msg "Updating www service IP"
  local existing_service=$(sudo kubectl \
    --kubeconfig=$KUBECONFIG \
    get services -o json \
    | jq '.items[] | select(.metadata.name == "www")')
  if [ -z "$existing_service" ]; then
    log_proc_error "www service not created in k8s cluster, exiting"
    exit 1
  else
    log_proc_msg "Found www service, updating IP address"
    local www_ip=$(echo "$existing_service" \
      | jq -r '.spec.clusterIP')
    set_config "WWW_IP" "$www_ip"
  fi
}

__expose_www_endpoint() {
  if [ ! -f "$KUBECONFIG" ]; then
    log_proc_error "No kubeconfig file present, exiting"
    exit 1
  fi

  local existing_service=$(sudo kubectl \
    --kubeconfig=$KUBECONFIG \
    get services -o json \
    | jq '.items[] | select(.metadata.name == "www")')
  if [ -z "$existing_service" ]; then
    log_proc_msg "Exposing service endpoint for www"
    local service_config="$MASTER_CONFIG_DIR/www_service.yml"
    kubectl --kubeconfig $KUBECONFIG \
      apply -f "$service_config"
  else
    log_proc_msg "WWW service already exists, skipping"
  fi
}

__generate_www_service() {
  log_proc_msg "Generating service config for www"

  local www_service_template="$INSTALLER_SCRIPTS_DIR/configs/www_service.yml.template"
  local www_service="$MASTER_CONFIG_DIR/www_service.yml"

  cp -vr "$www_service_template" "$www_service"

  sed -i "s#{{COMPONENT}}#www#g" $www_service
  sed -i "s#{{WWW_PORT}}#$WWW_PORT#g" $www_service
}

main() {
  log_proc_marker "Exposing service endpoints"
  if [ "$RUN_MODE" == "production" ]; then
    __generate_api_service
    __expose_api_endpoint
    __update_api_ip
    __generate_www_service
    __expose_www_endpoint
    __update_www_ip
  else
    log_proc_msg "Services not enabled for non-production mode, skipping"
  fi
}

main
