#!/bin/bash
set -e
set -o pipefail

# Logger methods ##########################################
###########################################################
log_proc_marker() {
  local prompt="$@"
  echo ""
  echo "# $(date +"%T") #######################################"
  echo "# $prompt"
  echo "##################################################"
}

log_proc_msg() {
  local message="$@"
  echo "|___ $@"
}

log_proc_error() {
  local message="$1"
  local error="$2"
  local bold_red_text='\e[91m'
  local reset_text='\033[0m'

  echo -e "$bold_red_text|___ $message$reset_text"
  echo -e "     $error"
}

log_proc_success() {
  local message="$@"
  local bold_green_text='\e[32m'
  local reset_text='\033[0m'

  echo -e "$bold_green_text|___ $message$reset_text"
}

# End Logger methods ######################################
###########################################################

export COMPONENT="msg"
export SCRIPTS_DIR="$SCRIPTS_DIR"
export LOGS_FILE="$RUNTIME_DIR/logs/$COMPONENT.log"
export SCRIPTS_DIR_REMOTE="/tmp/shippable"
export TIMEOUT=120

## Write logs of this script to component specific file
exec &> >(tee -a "$LOGS_FILE")

__validate_msg_envs() {
  log_proc_msg "Initializing vault environment variables"
  log_proc_msg "SCRIPTS_DIR: $SCRIPTS_DIR"
  log_proc_msg "RABBITMQ_IP: $RABBITMQ_IP"
  log_proc_msg "RABBITMQ_PORT: $RABBITMQ_PORT"
  log_proc_msg "RABBITMQ_ADMIN_PORT: $RABBITMQ_ADMIN_PORT"
  log_proc_msg "RABBITMQ_UI_USERNAME: $RABBITMQ_UI_USERNAME"
  log_proc_msg "RABBITMQ_UI_PASSWORD: ${#RABBITMQ_UI_PASSWORD}"
  log_proc_msg "RABBITMQ_USERNAME: $RABBITMQ_USERNAME"
  log_proc_msg "RABBITMQ_PASSWORD: ${#RABBITMQ_PASSWORD}"
  log_proc_msg "RABBITMQ_ADMIN: $RABBITMQ_ADMIN"
  log_proc_msg "LOGS_FILE: $LOGS_FILE"
}

__bootstrap_msg() {
  log_proc_msg "Bootstrapping RabbitMQ"
  #TODO: use rabbitmqAdapter in api for configuration

  log_proc_msg "Adding shippable user to RabbitMQ"
  local insert_amqp_user_res=$($RABBITMQ_ADMIN \
    --host=$RABBITMQ_IP --port=$RABBITMQ_ADMIN_PORT \
    declare user name=$RABBITMQ_USERNAME password=$RABBITMQ_PASSWORD tags=administrator)
  log_proc_msg "$insert_amqp_user_res"

  log_proc_msg  "Adding UI user to RabbitMQ"
  local insert_amqp_ui_user_res=$($RABBITMQ_ADMIN \
    --host=$RABBITMQ_IP --port=$RABBITMQ_ADMIN_PORT \
    declare user name=$RABBITMQ_UI_USERNAME password=$RABBITMQ_UI_PASSWORD tags=administrator)
  log_proc_msg "$insert_amqp_ui_user_res"

  log_proc_msg "Adding vhost: 'shippable to RabbitMQ"
  local insert_vhost_res=$($RABBITMQ_ADMIN \
    --host=$RABBITMQ_IP --port=$RABBITMQ_ADMIN_PORT \
    declare vhost name=shippable)
  log_proc_msg $insert_vhost_res

  log_proc_msg "Adding vhost: 'shippableRoot' to RabbitMQ"
  local insert_root_vhost_res=$($RABBITMQ_ADMIN \
    --host=$RABBITMQ_IP --port=$RABBITMQ_ADMIN_PORT \
    declare vhost name=shippableRoot)
  log_proc_msg $insert_root_vhost_res

  log_proc_msg "Updating user perms. $RABBITMQ_USERNAME -> shippable vhost"
  local update_user_perms_res=$($RABBITMQ_ADMIN \
    --host=$RABBITMQ_IP --port=$RABBITMQ_ADMIN_PORT \
    declare permission vhost=shippable user=$RABBITMQ_USERNAME configure=.* write=.* read=.*)
  log_proc_msg $update_user_perms_res

  log_proc_msg "Updating user perms. $RABBITMQ_USERNAME -> shippableRoot vhost"
  local update_user_perms_root_res=$($RABBITMQ_ADMIN \
    --host=$RABBITMQ_IP --port=$RABBITMQ_ADMIN_PORT \
    declare permission vhost=shippableRoot user=$RABBITMQ_USERNAME configure=.* write=.* read=.*)
  log_proc_msg $update_user_perms_root_res

  log_proc_msg "Updating user perms. $RABBITMQ_UI_USERNAME -> shippable vhost"
  local update_ui_user_perms_root_res=$($RABBITMQ_ADMIN \
    --host=$RABBITMQ_IP --port=$RABBITMQ_ADMIN_PORT \
    declare permission vhost=shippable user=$RABBITMQ_UI_USERNAME configure=.$ write=.* read=.*)
  log_proc_msg $update_ui_user_perms_root_res

  log_proc_msg "Updating user perms. $RABBITMQ_UI_USERNAME -> shippableRoot vhost"
  local update_ui_user_perms_root_res=$($RABBITMQ_ADMIN \
    --host=$RABBITMQ_IP --port=$RABBITMQ_ADMIN_PORT \
    declare permission vhost=shippableRoot user=$RABBITMQ_UI_USERNAME configure=.$ write=.* read=.*)
  log_proc_msg $update_ui_user_perms_root_res

  log_proc_msg "Adding exchange: 'shippableEx' to RabbitMQ for 'shippable' vhost"
  local insert_ex_res=$($RABBITMQ_ADMIN \
    --host=$RABBITMQ_IP --port=$RABBITMQ_ADMIN_PORT \
    --username=$RABBITMQ_USERNAME --password=$RABBITMQ_PASSWORD \
    --vhost=shippable declare exchange name=shippableEx type=topic)
  log_proc_msg $insert_ex_res

  log_proc_msg "Adding exchange: 'shippableEx' to RabbitMQ for 'shippableRoot' vhost"
  local insert_ex_root_res=$($RABBITMQ_ADMIN \
    --host=$RABBITMQ_IP --port=$RABBITMQ_ADMIN_PORT \
    --username=$RABBITMQ_USERNAME --password=$RABBITMQ_PASSWORD \
    --vhost=shippableRoot declare exchange name=shippableEx type=topic)
  log_proc_msg $insert_ex_root_res
}

main() {
  log_proc_marker "Initializing RabbitMQ "
  __validate_msg_envs
  __bootstrap_msg
}

main
