#!/bin/bash
set -e
set -o pipefail

export WELCOME_TEXT="
 _______  ___   _______  _______  ___      ___   __    _  _______  _______
|       ||   | |       ||       ||   |    |   | |  |  | ||       ||       |
|    _  ||   | |    _  ||    ___||   |    |   | |   |_| ||    ___||  _____|
|   |_| ||   | |   |_| ||   |___ |   |    |   | |       ||   |___ | |_____
|    ___||   | |    ___||    ___||   |___ |   | |  _    ||    ___||_____  |
|   |    |   | |   |    |   |___ |       ||   | | | |   ||   |___  _____| |
|___|    |___| |___|    |_______||_______||___| |_|  |__||_______||_______|

"
readonly UPDATE_TIME="$(date)"

__inject_defaults() {
  # TODO:
  # This needs to be removed when we start fetching project list from artifactory
  log_proc_marker "Injecting default data"

  log_proc_msg "Creating service user"
  _post_user
  if [ $is_request_success == true ]; then
    log_proc_success "Successfully created service user"
  else
    log_proc_error "Error while creating service user"
    log_proc_error "Status code: $response_status_code"
    exit 1
  fi

  log_proc_msg "Creating provider "
  local provider_data='{"name": "artifactory","url": "'$RT_URL'"}'
  _post_provider "$provider_data"
  if [ $is_request_success == true ]; then
    local provider_id=$(echo $response | jq -r '.id')
    log_proc_success "Successfully created provider: $provider_id "
  else
    log_proc_error "Error while creating provider"
    log_proc_error "Status code: $response_status_code"
    exit 1
  fi
  local provider_id="$(echo $response \
    | jq -r '.id')"

  log_proc_msg "Creating project"
  local project_data='{"name": "default_project","sourceId": "default_source_id","providerId": '$provider_id', "configPropertyBag": {}}'
  _post_project "$project_data"
  if [ $is_request_success == true ]; then
    local project_id=$(echo $response | jq -r '.id')
    log_proc_success "Successfully created project: $project_id "
  else
    log_proc_error "Error while creating project"
    log_proc_error "Status code: $response_status_code"
    exit 1
  fi
}

__update_last_update_time() {
  log_proc_msg "Installation complete"
  set_config "LAST_UPDATE_TIME" "$UPDATE_TIME"
}

__boot_services() {
  log_proc_msg "Booting services"
  local services=("pipelineSync" "runTrigger" "stepTrigger" "cron" "nexec")
  for service in ${services[@]}; do
    local service_data='{"name": "'$service'"}'

    _post_services "$service_data"
    if [ $is_request_success == true ]; then
      log_proc_success "$service started"
    else
      log_proc_error "Error while starting  $service"
      log_proc_error "Status code: $response_status_code"
      exit 1
    fi
  done
}

__boot_www() {
  log_proc_msg "Booting WWW"
  local service='{"name": "www"}'
  _post_services "$service"
  if [ $is_request_success == true ]; then
    log_proc_success "WWW started"
  else
    log_proc_error "Error while starting WWW "
    log_proc_error "Status code: $response_status_code \
      Log file: $LOGS_DIR/www.log"
    exit 1
  fi
}

__boot_api() {
  log_proc_msg "Booting API"
  local service='{"name": "api"}'
  _post_services "$service"
  if [ $is_request_success == true ]; then
    sleep 6
    log_proc_success "API started"
  else
    log_proc_error "Error while starting API"
    log_proc_error "Status code: $response_status_code \
      Log file: $LOGS_DIR/api.log"
    exit 1
  fi
}

__initialize_cluster() {
  log_proc_msg "Initializing kubernetes master"
  source "$INSTALLER_SCRIPTS_DIR/initializeMaster.sh"
}

__initialize_msg() {
  log_proc_msg "Initializing msg"
  _post_msg_initialize
  if [ $is_request_success == true ]; then
    log_proc_success "Msg initialized"
  else
    log_proc_error "Error while initializing msg"
    log_proc_error "Status code: $response_status_code"
    exit 1
  fi
}

__initialize_vault() {
  log_proc_msg "Initializing vault"
  _post_secrets_initialize
  if [ $is_request_success == true ]; then
    log_proc_success "Vault initialized"
  else
    log_proc_error "Error while initializing vault"
    log_proc_error "Status code: $response_status_code"
    exit 1
  fi
}

__initialize_db() {
  log_proc_msg "Initializing database (takes about a minute)"
  source "$INSTALLER_SCRIPTS_DIR/initializeDb.sh"
}

__boot_installer() {
  log_proc_msg "Booting installer container"
  platform_exec "boot_installer"
  # let installer initialize all routes
  sleep 10
}

__create_secrets() {
  log_proc_msg "Creating secrets to be used by deployments"
  source "$INSTALLER_SCRIPTS_DIR/createSecrets.sh"
}

__expose_endpoints() {
  log_proc_msg "Exposing external endpoints for services"
  source "$INSTALLER_SCRIPTS_DIR/exposeEndpoints.sh"
}

__install_redis() {
  log_proc_msg "Installing redis"
  source "$INSTALLER_SCRIPTS_DIR/installRedis.sh"
}

__install_msg() {
  log_proc_msg "Installing msg"
  source "$INSTALLER_SCRIPTS_DIR/installMsg.sh"
}

__install_vault() {
  log_proc_msg "Installing vault"
  source "$INSTALLER_SCRIPTS_DIR/installVault.sh"
}

__create_vault_table() {
  log_proc_msg "Creating vault table"
  source "$INSTALLER_SCRIPTS_DIR/createVaultTable.sh"
}

__install_db() {
  log_proc_msg "Installing database"
  source "$INSTALLER_SCRIPTS_DIR/installDb.sh"
}

__install_dependencies() {
  log_proc_marker "Installing dependencies"
  platform_exec "install_dependencies"
}

__set_release() {
  log_proc_msg "Set release version"
  local version_file="$INSTALLER_HOME/version.txt"
  if [ ! -f "$version_file" ]; then
    log_proc_error "No version.txt found, exiting"
    exit 1
  else
    local install_version=$(cat "$version_file")
    if [ ! -z "$install_version" ]; then
      set_config "RELEASE" "$install_version"
    else
      set_config "RELEASE" "master"
    fi
  fi
}

__pull_images() {
  log_proc_marker "Pulling images"
  platform_exec "pull_service_images"
}

__generate_service_id() {
  log_proc_msg "Generating service Id for the installation"
  if [ -z "$RT_SERVICE_ID" ]; then
    local rt_service_id="jfrt@$(date +%s)"
    set_config "RT_SERVICE_ID" "$rt_service_id"
  else
    log_proc_msg "Service ID already generated: $RT_SERVICE_ID"
  fi
}

__generate_ssh_keypair() {
  log_proc_msg "Generate ssh keypair"
  platform_exec "generate_ssh_keypair"
}

__get_artifactory_creds() {
  log_proc_msg "Getting artifactory credentials"

  if [ -z "$RT_URL" ]; then
    log_proc_success "Please enter the Artifactory connection URL"
    read response
    set_config "RT_URL" "$response"
  else
    log_proc_msg "RT_URL: $RT_URL"
  fi

  if [ -z "$RT_USERNAME" ]; then
    log_proc_success "Please enter the Artifactory admin username"
    read response
    set_config "RT_USERNAME" "$response"
  else
    log_proc_msg "RT_USERNAME: $RT_USERNAME"
  fi

  if [ -z "$RT_PASSWORD" ]; then
    log_proc_success "Please enter the Artifactory admin password"
    read response
    set_config "RT_PASSWORD" "$response"
  else
    log_proc_msg "RT_PASSWORD: $RT_PASSWORD"
  fi
}

__connect_to_artifactory() {
  log_proc_marker "Connecting with artifactory"

  _rt_ping_server
  if [ $is_request_success == true ] && [ "$response" == "OK" ]; then
    log_proc_msg "Successfully pinged artifactory instance"
  else
    log_proc_error "Error checking artifactory health. Return status: $response"
    log_proc_error "Status code: $response_status_code"
    exit 1
  fi
}

__validate_licenses() {
  log_proc_marker "Validating licenses with artifactory"
  local assigned_licenses=""
  _rt_get_licenses
  if [ $is_request_success == true ]; then
    log_proc_msg "Successfully fetched licenses from artifactory"
    assigned_licenses="$response"
    if [ -z "$assigned_licenses" ]; then
      log_proc_error "No licenses assigned to artifactory instance, exiting"
      exit 1
    fi
  else
    log_proc_error "Error getting licenses: $response"
    log_proc_error "Status code: $response_status_code"
    exit 1
  fi

  local license_type_e_plus_trial="Enterprise Plus Trial"
  local license_type_e_plus="Enterprise Plus"

  local assigned_license=$(echo $assigned_licenses \
    | jq -r '.type')
  if [ ! -z "$assigned_license" ] && [ "$assigned_license" != "null" ]; then
    # parse single object format
    if [ "$assigned_license" == "$license_type_e_plus_trial" ]; then
      log_proc_success "License type: $assigned_license"
    elif [ "$assigned_license" == "$license_type_e_plus" ]; then
      log_proc_success "License type: $assigned_license"
    else
      log_proc_error "Invalid license type: $assigned_license"
      exit 1
    fi
  else
    # parse array format
    local assigned_licenses_e_plus_trial=$(echo $assigned_licenses \
      | jq '.licenses[] | select(.type=="'$license_type_e_plus_trial'")')
    local assigned_licenses_e_plus=$(echo $assigned_licenses \
      | jq '.licenses[] | select(.type=="'$license_type_e_plus'")')

    if [ ! -z "$assigned_licenses_e_plus_trial" ]; then
      log_proc_success "License type: $license_type_e_plus_trial"
    fi

    if [ ! -z "$assigned_licenses_e_plus" ]; then
      log_proc_success "License type: $license_type_e_plus"
    fi

    if [ -z "$license_type_e_plus_trial" ] && [ -z "$license_type_e_plus" ]; then
      log_proc_error "Invalid license assigned to Artifactory"
      log_proc_error "Artifactory needs either Enterprise Plus or Enterprise Plus Trial license"
      exit 1
    fi

  fi
}

__configure_state() {
  log_proc_msg "Configuring state"
  source "$INSTALLER_SCRIPTS_DIR/configureState.sh"
}

__configure_proxy() {
  log_proc_msg "Configuring proxy settings"
  source "$INSTALLER_SCRIPTS_DIR/configureProxy.sh"
}

__get_proxy_config() {
  log_proc_msg "Getting proxy settings from artifactory"

  if [ ! -z "$PIPELINES_HTTP_PROXY" ] || [ ! -z "$PIPELINES_HTTPS_PROXY" ]; then
    log_proc_msg "Proxy already set from CLI, skipping artifactory proxy check"
    return
  fi

  local proxies=""
  _rt_get_proxies
  if [ $is_request_success == true ]; then
    log_proc_msg "Successfully fetched proxy configuration from artifactory"
    if [ -z "$response" ]; then
      log_proc_warn "No proxy configurations fetched from artifactory instance"
    else
      proxies="$response"
    fi
  elif [ "$response_status_code" -eq 404 ]; then
    log_proc_warn "Artifactory does not support the latest proxy API, skipping proxy config"
    return
  else
    log_proc_error "Error getting proxy configuration: $response"
    log_proc_error "Status code: $response_status_code"
    exit 1
  fi

  local default_proxy=$(echo $proxies \
    | jq '.proxies[] | select(.default_proxy==true)')
  if [ -z "$default_proxy" ]; then
    log_proc_msg "No proxy configured as global default, skipping proxy configuration"
  else
    local proxy_host=$(echo $default_proxy \
      | jq -r '.host')
    local proxy_port=$(echo $default_proxy \
      | jq -r '.port')
    local proxy_url="http://$proxy_host:$proxy_port"

    local proxy_username=$(echo $default_proxy \
      | jq -r '.username')
    if [ ! -z "$proxy_username" ]; then
      set_config "PIPELINES_PROXY_USERNAME" "$proxy_username"
    fi

    local proxy_password=$(echo $default_proxy \
      | jq -r '.password')
    if [ ! -z "$proxy_password" ]; then
      set_config "PIPELINES_PROXY_PASSWORD" "$proxy_password"
    fi

    set_config "PIPELINES_HTTP_PROXY" "$proxy_url"
    set_config "PIPELINES_HTTPS_PROXY" "$proxy_url"
  fi
}

__get_mailserver_config() {
  log_proc_marker "Configuring mailserver"
  log_proc_msg "Getting mailserver settings from artifactory"

  local mailservers=""
  _rt_get_mailserver
  if [ $is_request_success == true ]; then
    log_proc_msg "Successfully fetched mailserver config from artifactory"
    if [ -z "$response" ]; then
      log_proc_warn "No mailserver config fetched from artifactory instance"
    else
      mailservers="$response"
    fi
  elif [ "$response_status_code" -eq 404 ]; then
    log_proc_warn "Artifactory does not support the latest mailserver API, skipping mailserver config"
    return
  elif [ "$response_status_code" -eq 500 ]; then
    # NOTE: this needs to be ignore because of an existing bug in artifactory
    log_proc_warn "Internal server error when getting latest mailserver, skipping mailserver config "
    return
  else
    log_proc_error "Error getting mailserver configuration: $response"
    log_proc_error "Status code: $response_status_code"
    exit 1
  fi

  local mailserver_config=$(echo $mailservers \
    | jq '.')
  local mailserver_enabled=$(echo $mailserver_config \
    | jq -r '.enabled')

  if [ ! -z "$mailserver_config" ] && [ "$mailserver_enabled" == "true" ]; then
    log_proc_msg "Found mailserver configuration in artifactory"
    local mailserver_username=$(echo $mailserver_config \
      | jq -r '.username')
    local mailserver_password=$(echo $mailserver_config \
      | jq -r '.password')
    local mailserver_host=$(echo $mailserver_config \
      | jq -r '.host')
    local mailserver_port=$(echo $mailserver_config \
      | jq -r '.port')
    local mailserver_tls=$(echo $mailserver_config \
      | jq -r '.tls')
    local mailserver_ssl=$(echo $mailserver_config \
      | jq -r '.ssl')

    set_config "MAILSERVER_USERNAME" "$mailserver_username"
    set_config "MAILSERVER_PASSWORD" "$mailserver_password"
    set_config "MAILSERVER_HOST" "$mailserver_host"
    set_config "MAILSERVER_PORT" "$mailserver_port"
    set_config "MAILSERVER_TLS" "$mailserver_tls"
    set_config "MAILSERVER_SSL" "$mailserver_ssl"
  elif [ ! -z "$mailserver_config" ] && [ "$mailserver_enabled" == "false" ]; then
    log_proc_warn "Mailserver disabled, skipping configuration"
  else
    log_proc_warn "No mailserver settings found in artifactory, skipping configuration"
  fi
}

__set_db_ip() {
  log_proc_success "Please enter the URL of the database"
  read response
  if [ "$response" == "localhost" ] || [ "$response" == "127.0.0.1" ]; then
    set_config "DB_EXTERNAL_URL" "$INSTALLER_IP"
  elif validate_ip $response ; then
    set_config "DB_EXTERNAL_URL" "$response"
  else
    log_proc_error "Invalid response, please enter valid IP address"
    __set_db_ip
  fi
}

__set_db_password() {
  log_proc_msg "Setting database password"

  log_proc_success "Please enter the password for your database."
  read response

  if [ "$response" == "" ]; then
    log_proc_error "Database password cannot be empty. Re-enter password"
    __set_db_password
  elif validate_db_password $response ; then
    set_config "DB_PASSWORD" "$response"
  else
    log_proc_error "Invalid characters in the database password field, valid characters are: -_=+a-zA-Z0-9"
    __set_db_password
  fi
}

__set_defaults() {
  if [ -z "$RUN_MODE" ]; then
    set_config "RUN_MODE" "onebox"
  fi

  if [ -z "$INSTALL_USER" ]; then
    local user=$(id -u)
    local group=$(id -g)
    if [ -z "$user" ]; then
      log_proc_error "Cannot set installation user, empty value returned from 'id -u' "
      exit 1
    else
      set_config "INSTALL_USER" "$user"
      set_config "INSTALL_GROUP" "$group"
    fi
  fi

  if [ -z "$INSTALLER_AUTH_TOKEN" ]; then
    if [ -z "$GLOBAL_PASSWORD" ]; then
      export GLOBAL_PASSWORD=$(uuidgen)
    fi
    set_config "INSTALLER_AUTH_TOKEN" "$GLOBAL_PASSWORD"
  else
    export GLOBAL_PASSWORD="$INSTALLER_AUTH_TOKEN"
  fi

  # If user sets rabbitmq external URL, set following fields from its parsed values
  # - username
  # - password
  # NOTE: using inline python to parse urls for a cleaner solution. regexp handling
  # can get really complicated and unreadable in native bash
  local python_path=$(which python)
  if [ -z "$python_path" ]; then
    log_proc_error "Python not found, exiting"
    exit 1
  fi

  if [ ! -z "$RABBITMQ_EXTERNAL_URL" ]; then
    local protocol=$($python_path -c \
      'import os; from urlparse import urlparse; result=urlparse("'$RABBITMQ_EXTERNAL_URL'"); print(result.scheme)')

    if [ -z "$protocol" ]; then
      log_proc_error "Missing protocol for RabbitMQ Url. Usage: --rabbitmq-url amqp[s]://user:pass@host[:port]"
      exit 1
    elif [ "$protocol" != "amqp" ] && [ "$protocol" != "amqps" ]; then
      log_proc_error "Incorrect protocol for RabbitMQ Url. Valid protocols: amqp/amqps"
      exit 1
    fi

    local rabbitmq_username=$($python_path -c \
      'import os; from urlparse import urlparse; result=urlparse("'$RABBITMQ_EXTERNAL_URL'"); print(result.username)')
    local rabbitmq_password=$($python_path -c \
      'import os; from urlparse import urlparse; result=urlparse("'$RABBITMQ_EXTERNAL_URL'"); print(result.password)')

    if [ ! -z "$rabbitmq_username" ] && [ "$rabbitmq_username" != "None" ]; then
      if [ ! -z "$rabbitmq_password" ] && [ "$rabbitmq_password" != "None" ]; then
        set_config "RABBITMQ_USERNAME" "$rabbitmq_username"
        set_config "RABBITMQ_PASSWORD" "$rabbitmq_password"
      else
        log_proc_msg "No password set for rabbitmq admin, using default username and password"
      fi
    else
      log_proc_msg "No username set for rabbitmq admin, using default username and password"
    fi
  fi

  # If user sets rabbitmq admin URL, set following fields from its parsed values
  # - UI username
  # - UI user password
  if [ ! -z "$RABBITMQ_EXTERNAL_ADMIN_URL" ]; then
    local protocol=$($python_path -c \
      'import os; from urlparse import urlparse; result=urlparse("'$RABBITMQ_EXTERNAL_ADMIN_URL'"); print(result.scheme)')

    if [ -z "$protocol" ]; then
      log_proc_error "Missing protocol for RabbitMQ Admin Url. Usage: --rabbitmq-admin-url http[s]://user:pass@host[:port]"
      exit 1
    elif [ "$protocol" != "http" ] && [ "$protocol" != "https" ]; then
      log_proc_error "Incorrect protocol for RabbitMQ Admin Url. Valid protocols: http/https"
      exit 1
    fi

    local rabbitmq_ui_username=$($python_path -c \
      'import os; from urlparse import urlparse; result=urlparse("'$RABBITMQ_EXTERNAL_ADMIN_URL'"); print(result.username)')
    local rabbitmq_ui_password=$($python_path -c \
      'import os; from urlparse import urlparse; result=urlparse("'$RABBITMQ_EXTERNAL_ADMIN_URL'"); print(result.password)')

    if [ ! -z "$rabbitmq_ui_username" ] && [ "$rabbitmq_ui_username" != "None" ]; then
      if [ ! -z "$rabbitmq_ui_password" ] && [ "$rabbitmq_ui_password" != "None" ]; then
        set_config "RABBITMQ_UI_USERNAME" "$rabbitmq_ui_username"
        set_config "RABBITMQ_UI_PASSWORD" "$rabbitmq_ui_password"
      else
        log_proc_msg "No password set for rabbitmq UI, using default username and password"
      fi
    else
      log_proc_msg "No username set for rabbitmq UI, using default username and password"
    fi
  fi

  if [ "$RUN_MODE" == "onebox" ]; then
    # installer ip is required during onebox install
    if [ -z "$INSTALLER_IP" ]; then
      log_proc_error "Installer IP cannot be empty in onebox mode"
      log_proc_error "Please set --installer-ip <internal_ip> argument during installation"
      exit 1
    fi
    # disallow localhost because kubernetes containers cannot access these
    if [ "$INSTALLER_IP" == '127.0.0.1' ] || [ "$INSTALLER_IP" == 'localhost' ]; then
      log_proc_error "Installer IP cannot be set to 127.0.0.1 or localhost"
      log_proc_error "Please set the external IP address of the installation host"
      exit 1
    fi
    set_config "DB_EXTERNAL_URL" "$INSTALLER_IP"
    set_config "DB_IP" "$INSTALLER_IP"
    set_config "VAULT_IP" "$INSTALLER_IP"
    set_config "RABBITMQ_IP" "$INSTALLER_IP"
    set_config "REDIS_IP" "$INSTALLER_IP"
    set_config "API_IP" "$INSTALLER_IP"
    set_config "WWW_IP" "$INSTALLER_IP"
  fi

  # production mode
  if [ "$RUN_MODE" != "devmode" ] && [ "$RUN_MODE" != "onebox" ]; then
    # installer ip is required during onebox install
    if [ -z "$INSTALLER_IP" ]; then
      log_proc_error "Installer IP cannot be empty in HA mode"
      log_proc_error "Please set --installer-ip <internal_ip> argument during installation"
      exit 1
    fi
    # disallow localhost because kubernetes containers cannot access these
    if [ "$INSTALLER_IP" == '127.0.0.1' ] || [ "$INSTALLER_IP" == 'localhost' ]; then
      log_proc_error "Installer IP cannot be set to 127.0.0.1 or localhost"
      log_proc_error "Please set the external IP address of the installation host"
      exit 1
    fi

    if [ -z "$DB_EXTERNAL_URL" ]; then
      __set_db_ip
    fi
    set_config "DB_IP" "$DB_EXTERNAL_URL"

    if [ -z "$DB_PASSWORD" ]; then
      __set_db_password
    fi
  fi

  if [ -z "$ROOT_BUCKET" ]; then
    local bucket_name="pipelines_$(date +%s)"
    set_config "ROOT_BUCKET" "$bucket_name"
  fi

  local default_registry_creds="$HOME/.docker/config.json"
  if [ ! -f "$REGISTRY_CREDS" ]; then
    if [ -f "$default_registry_creds" ]; then
      log_proc_warn "Using default registry credentials: $default_registry_creds"
      cp -vr "$default_registry_creds" "$REGISTRY_CREDS"
    fi
  fi

  local default_registry_url="docker.bintray.io"
  if [ -z "$IMAGE_REGISTRY_URL" ]; then
    if [ ! -f "$REGISTRY_CREDS" ]; then
      set_config "IMAGE_REGISTRY_URL" "$default_registry_url"
    else
      # using the first registry from docker config by default
      local registry_url=$(cat $REGISTRY_CREDS \
        | jq -r '.auths | keys[0]')
      if [ ! -z "$registry_url" ]; then
        set_config "IMAGE_REGISTRY_URL" "$registry_url"
      fi
    fi
  fi

  local default_kube_config="$HOME/.kube/config"
  if [ ! -f "$KUBECONFIG" ]; then
    if [ -f "$default_kube_config" ] && [ -r "$default_kube_config" ]; then
      log_proc_warn "Using default kube config: $default_kube_config"
      cp $default_kube_config $KUBECONFIG
    elif [ -f "$default_kube_config" ] && [ ! -r "$default_kube_config" ]; then
      log_proc_error "Cannot read default kube config: $default_kube_config"
      exit 1
    fi
  fi

  ## fallback values
  if [ -z "$DB_PASSWORD" ]; then
    set_config "DB_PASSWORD" "$GLOBAL_PASSWORD"
  fi
  if [ -z "$RABBITMQ_PASSWORD" ]; then
    set_config "RABBITMQ_PASSWORD" "$GLOBAL_PASSWORD"
  fi
  if [ -z "$RABBITMQ_UI_PASSWORD" ]; then
    set_config "RABBITMQ_UI_PASSWORD" "$GLOBAL_PASSWORD"
  fi
}

__validate_run_mode() {
  if [ "$#" -ne 1 ]; then
    log_proc_error "No RUN_MODE provided for validation, exiting"
    exit 1
  fi

  local new_run_mode="$1"
  if [ ! -z "$RUN_MODE" ] && [ "$RUN_MODE" != "$new_run_mode" ]; then
    log_proc_msg "Cannot change RUN_MODE of an existing installation"
    log_proc_msg "To change RUN_MODE, clean up the current installation and install again or manually change it"
  fi
}

__validate_user() {
  local user_grps="$1"
  local install_user=$(echo $user_grps | cut -d':' -f1)
  local install_group=$(echo $user_grps | cut -d':' -f2)

  if [ -z "$install_group" ]; then
    install_group="$install_user"
  fi

  ## Only save the ID's of user and group
  install_user=$(id -u $install_user)
  install_group=$(id -g $install_group)

  set_config "INSTALL_USER" "$install_user"
  set_config "INSTALL_GROUP" "$install_group"
}

__parse_args() {
  if [[ $# -gt 0 ]]; then
    while [[ $# -gt 0 ]]; do
      ARGUMENT="$1"

      case $ARGUMENT in
        -s|--silent)
          export NO_PROMPT=true
          shift
          ;;
        -d|--devmode)
          __validate_run_mode "devmode"
          log_proc_msg "Running installer in devmode"
          set_config "RUN_MODE" "devmode"
          set_config "INSTALLER_IP" "127.0.0.1"
          set_config "DB_EXTERNAL_URL" "$INSTALLER_IP"
          set_config "DB_IP" "$INSTALLER_IP"
          set_config "VAULT_IP" "$INSTALLER_IP"
          set_config "RABBITMQ_IP" "$INSTALLER_IP"
          set_config "REDIS_IP" "$INSTALLER_IP"
          set_config "API_IP" "$INSTALLER_IP"
          set_config "WWW_IP" "$INSTALLER_IP"
          shift
          ;;
        --installer-ip)
          local installer_ip="$2"
          set_config "INSTALLER_IP" "$installer_ip"
          shift
          shift
          ;;
        --global-password)
          if [ -z "$2" ]; then
            log_proc_error "Global password cannot be empty"
            exit 1
          else
            export GLOBAL_PASSWORD="$2"
          fi
          shift
          shift
          ;;
        --install-user)
          local install_user="$2"
          if [[ $install_user == --* ]] || [ -z "$install_user" ]; then
            log_proc_error "Install user cannot be empty"
            exit 1
          else
            __validate_user "$install_user"
          fi
          shift
          shift
          ;;
        --rt-url)
          local rt_url="$2"
          set_config "RT_URL" "$rt_url"
          shift
          shift
          ;;
        --rt-username)
          local rt_username="$2"
          set_config "RT_USERNAME" "$rt_username"
          shift
          shift
          ;;
        --rt-service-id)
          local rt_service_id="$2"
          set_config "RT_SERVICE_ID" "$rt_service_id"
          shift
          shift
          ;;
        --rt-password)
          local rt_password="$2"
          set_config "RT_PASSWORD" "$rt_password"
          shift
          shift
          ;;
        --rt-proxy)
          local rt_proxy="$2"
          set_config "RT_PROXY" "$rt_proxy"
          shift
          shift
          ;;
        --rt-proxy-username)
          local rt_proxy_username="$2"
          set_config "RT_PROXY_USERNAME" "$rt_proxy_username"
          shift
          shift
          ;;
        --rt-proxy-password)
          local rt_proxy_password="$2"
          set_config "RT_PROXY_PASSWORD" "$rt_proxy_password"
          shift
          shift
          ;;
        --image-registry-url)
          local registry_url="$2"
          set_config "IMAGE_REGISTRY_URL" "$registry_url"
          shift
          shift
          ;;
        --image-registry-creds)
          local registry_creds_file="$2"
          if [ -f "$registry_creds_file" ]; then
            cp -r "$registry_creds_file" "$REGISTRY_CREDS"
          else
            log_proc_error "Missing registry credentials file: $registry_creds_file"
            exit 1
          fi
          shift
          shift
          ;;
        --www-url)
          local www_url="$2"
          set_config "WWW_EXTERNAL_URL" "$www_url"
          shift
          shift
          ;;
        --api-url)
          local api_url="$2"
          set_config "API_EXTERNAL_URL" "$api_url"
          shift
          shift
          ;;
        --skip-image-pull)
          export SKIP_IMAGE_PULL=true
          shift
          ;;
        --rabbitmq-url)
          local rabbitmq_external_url="$2"
          local rabbitmq_server_url="$rabbitmq_external_url"/shippable
          set_config "RABBITMQ_EXTERNAL_URL" "$rabbitmq_server_url"

          local rabbitmq_root_url="$rabbitmq_external_url"/shippableRoot
          set_config "RABBITMQ_EXTERNAL_ROOT_URL" "$rabbitmq_root_url"

          shift
          shift
          ;;
        --rabbitmq-admin-url)
          local rabbitmq_admin_url="$2"
          set_config "RABBITMQ_EXTERNAL_ADMIN_URL" "$rabbitmq_admin_url"
          shift
          shift
          ;;
        --state-bucket)
          local state_bucket="$2"
          set_config "ROOT_BUCKET" "$state_bucket"
          shift
          shift
          ;;
        --global-http-proxy)
          local global_http_proxy="$2"
          set_config "PIPELINES_HTTP_PROXY" "$global_http_proxy"
          shift
          shift
          ;;
        --global-https-proxy)
          local global_https_proxy="$2"
          set_config "PIPELINES_HTTPS_PROXY" "$global_https_proxy"
          shift
          shift
          ;;
        --global-proxy-username)
          local global_proxy_username="$2"
          set_config "PIPELINES_PROXY_USERNAME" "$global_proxy_username"
          shift
          shift
          ;;
        --global-proxy-password)
          local global_proxy_password="$2"
          set_config "PIPELINES_PROXY_PASSWORD" "$global_proxy_password"
          shift
          shift
          ;;
        --global-no-proxy)
          local global_no_proxy="$2"
          set_config "PIPELINES_NO_PROXY" "$global_no_proxy"
          shift
          shift
          ;;
        --config-backup-count)
          local config_backup_count="$2"
          set_config "CONFIG_BACKUP_COUNT" "$config_backup_count"
          shift
          shift
          ;;
        *)
          echo "Invalid option: $ARGUMENT"
          print_help
          ;;
      esac
    done
  fi
  __set_defaults
}

main() {
  log_proc_marker "Installing pipelines"
  echo "$WELCOME_TEXT"
  __set_release
  __install_dependencies
  __parse_args "$@"

  #validations and configuration
  __generate_ssh_keypair
  __get_artifactory_creds
  __generate_service_id
  __connect_to_artifactory
  __validate_licenses
  __get_proxy_config
  __get_mailserver_config
  __configure_proxy
  __configure_state

  # Setup environment
  __pull_images
  __create_secrets
  __install_db
  __create_vault_table
  __initialize_cluster
  __install_vault
  __install_msg
  __install_redis
  __expose_endpoints

  # Initialize core components
  __boot_installer
  __initialize_db
  __initialize_vault
  __initialize_msg
  __inject_defaults

  # Initialize services
  __boot_api
  __boot_www
  __boot_services

  __update_last_update_time
}

{
  main "$@"
} 2>&1 | tee "$LOG_FILE" ; ( exit ${PIPESTATUS[0]} )
