#!/bin/bash
set -e
set -o pipefail

export COMPONENT="db"
export DB_DATA_DIR="$RUNTIME_DIR/$COMPONENT/data"
export DB_CONFIG_DIR="$CONFIG_DIR/$COMPONENT"
export DB_INSTALL_SCRIPT="installDb.sh"

__validate_db_envs() {
  log_proc_msg "Validating db ENV variables"
  if [ "$DB_IP" == "" ]; then
    log_proc_error "DB_IP cannot be empty, exiting"
    exit 1
  fi

  if [ "$DB_PORT" == "" ]; then
    log_proc_error "DB_PORT cannot be empty, exiting"
    exit 1
  fi

  if [ "$DB_USER" == "" ]; then
    log_proc_error "DB_USER cannot be empty, exiting"
    exit 1
  fi

  if [ "$DB_NAME" == "" ]; then
    log_proc_error "DB_NAME cannot be empty, exiting"
    exit 1
  fi

  if [ "$DB_PASSWORD" == "" ]; then
    log_proc_error "DB_PASSWORD cannot be empty, exiting"
    exit 1
  fi

  if [ "$DB_DIALECT" == "" ]; then
    log_proc_error "DB_DIALECT cannot be empty, exiting"
    exit 1
  fi
}

__validate_db_mounts() {
  log_proc_msg "Validating db data mounts from host"

  if [ ! -d "$DB_DATA_DIR" ]; then
    log_proc_msg "Creating data directory $DB_DATA_DIR"
    mkdir -p $DB_DATA_DIR
  else
    log_proc_msg "Data directory already present: $DB_DATA_DIR"
  fi

  if [ ! -d "$DB_CONFIG_DIR" ]; then
    log_proc_msg "Creating config directory $DB_CONFIG_DIR"
    mkdir -p $DB_CONFIG_DIR
  else
    log_proc_msg "Config directory already present: $DB_CONFIG_DIR"
  fi
}

__test_db_connection() {
  local create_table_cmd="PGPASSWORD=$DB_PASSWORD \
    psql \
    -U $DB_USER \
    -d $DB_NAME \
    -h $DB_IP \
    -p $DB_PORT \
    -v ON_ERROR_STOP=1 \
    -c 'CREATE TABLE \"testTable\" (\"testColumn\" integer PRIMARY KEY);'"
  local drop_table_cmd="PGPASSWORD=$DB_PASSWORD \
    psql \
    -U $DB_USER \
    -d $DB_NAME \
    -h $DB_IP \
    -p $DB_PORT \
    -v ON_ERROR_STOP=1 \
    -c 'DROP TABLE \"testTable\";'"
    log_proc_msg "Creating a test table to verify database connection"
    eval "$create_table_cmd"
    log_proc_msg "Dropping a test table"
    eval "$drop_table_cmd"
    log_proc_msg "Successfully tested connection to $DB_IP $DB_PORT"
}

__install_db_compose() {
  source "$INSTALLER_SCRIPTS_DIR/compose/$DB_INSTALL_SCRIPT"
}

__install_db_kubernetes() {
  source "$INSTALLER_SCRIPTS_DIR/kubernetes/$DB_INSTALL_SCRIPT"
}

main() {
  log_proc_marker "Installing database"
  __validate_db_envs
  __validate_db_mounts
  if [ "$RUN_MODE" == "devmode" ] || [ "$RUN_MODE" == "onebox" ]; then
    log_proc_msg "Installing database using docker compose"
    __install_db_compose
    __test_db_connection
  else
    log_proc_msg "Connecting to existing database"
    log_proc_msg "DB_IP: $DB_IP"
    log_proc_msg "DB_USER: $DB_USER"
    log_proc_msg "DB_NAME: $DB_NAME"
    __install_db_kubernetes
    __test_db_connection
  fi
}

main
