#!/bin/bash
set -e
set -o pipefail

export MSG_DEPLOYMENT="msg-deployment"
export MSG_DEPLOYMENT_CONFIG="$MSG_CONFIG_DIR/msg_deployment.yml"
export MSG_SERVICE_CONFIG="$MSG_CONFIG_DIR/msg_service.yml"

__update_msg_ip() {
  log_proc_msg "Updating msg service IP"
  local existing_service=$(kubectl \
    --kubeconfig=$KUBECONFIG \
    get services -o json \
    | jq '.items[] | select(.metadata.name == "msg")')
  if [ -z "$existing_service" ]; then
    log_proc_error "msg service not created in k8s cluster, exiting"
    exit 1
  else
    log_proc_msg "Found msg service, updating IP address"
    local msg_ip=$(echo "$existing_service" \
      | jq -r '.spec.clusterIP')
    set_config "RABBITMQ_IP" "$msg_ip"
  fi
}

__create_service() {
  log_proc_msg "Creating service for RabbitMQ"

  local existing_service=$(kubectl \
    --kubeconfig=$KUBECONFIG \
    get services -o json \
    | jq '.items[] | select(.metadata.name == "msg")')
  if [ -z "$existing_service" ]; then
    local rabbitmq_service_template="$INSTALLER_SCRIPTS_DIR/configs/rabbitmq_service.yml.template"

    cp -vr "$rabbitmq_service_template" "$MSG_SERVICE_CONFIG"
    sed -i "s#{{RABBITMQ_IP}}#$RABBITMQ_IP#g"  $MSG_SERVICE_CONFIG
    sed -i "s#{{RABBITMQ_PORT}}#$RABBITMQ_PORT#g" $MSG_SERVICE_CONFIG
    sed -i "s#{{RABBITMQ_ADMIN_PORT}}#$RABBITMQ_ADMIN_PORT#g" $MSG_SERVICE_CONFIG
    kubectl \
      --kubeconfig=$KUBECONFIG \
      create -f "$MSG_SERVICE_CONFIG"
  else
    log_proc_msg "Service already exists for RabbitMQ, skipping"
  fi
}

__cleanup_service() {
  log_proc_msg "Cleaning up configmap for msg"

  kubectl --kubeconfig $KUBECONFIG \
    delete service msg || true
}

__create_deployment() {
  log_proc_msg "Creating deployment"

  if [ ! -f "$MSG_DEPLOYMENT_CONFIG" ]; then
    log_proc_error "Deploy config missing for msg: $MSG_DEPLOYMENT_CONFIG"
    exit 1
  else
    log_proc_msg "Deploy config found, creating a new deployment"
    kubectl --kubeconfig $KUBECONFIG \
      apply -f "$MSG_DEPLOYMENT_CONFIG"
  fi
}

__cleanup_deployment() {
  log_proc_msg "Cleaning up deployment"

  kubectl --kubeconfig $KUBECONFIG \
    delete deployments "$MSG_DEPLOYMENT" || true
}

__update_deployment_config() {

  local msg_replicas=1
  local deployment_template="$INSTALLER_SCRIPTS_DIR/configs/rabbitmq_deployment.yml.template"
  cp -vr $deployment_template $MSG_DEPLOYMENT_CONFIG

  log_proc_msg "Updating replicas"
  sed -i 's#{{DEPLOYMENT_NAME}}#'$MSG_DEPLOYMENT'#g' $MSG_DEPLOYMENT_CONFIG

  log_proc_msg "Updating image registry secret"
  sed -i 's#{{IMAGE_REGISTRY_SECRET}}#'$IMAGE_REGISTRY_SECRET'#g' $MSG_DEPLOYMENT_CONFIG

  log_proc_msg "Updating replicas"
  sed -i 's#{{REPLICAS}}#'$msg_replicas'#g' $MSG_DEPLOYMENT_CONFIG

  log_proc_msg "Updating image name"
  sed -i 's#{{MSG_IMAGE}}#'$MSG_IMAGE'#g' $MSG_DEPLOYMENT_CONFIG

  log_proc_msg "Updating port"
  sed -i 's#{{MSG_PORT}}#'$RABBITMQ_PORT'#g' $MSG_DEPLOYMENT_CONFIG

  log_proc_msg "Updating admin port"
  sed -i 's#{{MSG_ADMIN_PORT}}#'$RABBITMQ_ADMIN_PORT'#g' $MSG_DEPLOYMENT_CONFIG

  log_proc_msg "Updating volume mount"
  sed -i 's#{{MSG_DATA_DIR}}#'$MSG_DATA_DIR'#g' $MSG_DEPLOYMENT_CONFIG

  log_proc_msg "Updating volume mount"
  sed -i 's#{{MSG_CONFIG_DIR}}#'$MSG_CONFIG_DIR'#g' $MSG_DEPLOYMENT_CONFIG
}

__check_msg() {
  log_proc_msg "Checking rabbitmq container status on: $RABBITMQ_IP:$RABBITMQ_PORT"

  platform_exec "check_service_connection" "$RABBITMQ_IP" "$RABBITMQ_PORT" "$COMPONENT"
  platform_exec "check_service_connection" "$RABBITMQ_IP" "$RABBITMQ_ADMIN_PORT" "MSG_ADMIN"
}

main() {
  __update_deployment_config
  __cleanup_deployment
  __create_deployment
  __cleanup_service
  __create_service
  __update_msg_ip
  __check_msg
}

main
