#!/bin/bash
set -e
set -o pipefail

export VAULT_CONFIGMAP="vault-config"
export VAULT_DEPLOYMENT="vault-deployment"
export VAULT_DEPLOYMENT_CONFIG="$VAULT_CONFIG_DIR/vault_deployment.yml"
export VAULT_SERVICE_CONFIG="$VAULT_CONFIG_DIR/vault_service.yml"

__update_vault_ip() {
  log_proc_msg "Updating vault service IP"
  local existing_service=$(kubectl \
    --kubeconfig=$KUBECONFIG \
    get services -o json \
    | jq '.items[] | select(.metadata.name == "vault")')
  if [ -z "$existing_service" ]; then
    log_proc_error "vault service not created in k8s cluster, exiting"
    exit 1
  else
    log_proc_msg "Found vault service, updating IP address"
    local vault_ip=$(echo "$existing_service" \
      | jq -r '.spec.clusterIP')
    set_config "VAULT_IP" "$vault_ip"
  fi
}

__create_service() {
  log_proc_msg "Creating service for Vault"

  local vault_service_template="$INSTALLER_SCRIPTS_DIR/configs/vault_service.yml.template"
  cp -vr $vault_service_template $VAULT_SERVICE_CONFIG

  sed -i "s#{{VAULT_IP}}#$VAULT_IP#g" $VAULT_SERVICE_CONFIG
  sed -i "s#{{VAULT_PORT}}#$VAULT_PORT#g" $VAULT_SERVICE_CONFIG
  kubectl \
    --kubeconfig=$KUBECONFIG \
    create -f "$VAULT_SERVICE_CONFIG"
}

__cleanup_service() {
  log_proc_msg "Cleaning up configmap for vault"

  kubectl --kubeconfig $KUBECONFIG \
    delete service vault || true
}

__create_deployment() {
  log_proc_msg "Creating vault deployment"

  if [ ! -f "$VAULT_DEPLOYMENT_CONFIG" ]; then
    log_proc_error "Deploy config missing for vault: $VAULT_DEPLOYMENT_CONFIG"
    exit 1
  else
    log_proc_msg "Deploy config found, creating a new deployment"
    kubectl --kubeconfig $KUBECONFIG \
      apply -f "$VAULT_DEPLOYMENT_CONFIG"
  fi
}

__cleanup_deployment() {
  log_proc_msg "Cleaning up deployment vault"

  kubectl --kubeconfig $KUBECONFIG \
    delete deployments "$VAULT_DEPLOYMENT" || true
}

__update_deployment_config() {

  local vault_replicas=1
  local deployment_template="$INSTALLER_SCRIPTS_DIR/configs/vault_deployment.yml.template"
  cp -vr $deployment_template $VAULT_DEPLOYMENT_CONFIG

  log_proc_msg "Updating replicas"
  sed -i 's#{{DEPLOYMENT_NAME}}#'$VAULT_DEPLOYMENT'#g' $VAULT_DEPLOYMENT_CONFIG

  log_proc_msg "Updating image registry secret"
  sed -i 's#{{IMAGE_REGISTRY_SECRET}}#'$IMAGE_REGISTRY_SECRET'#g' $VAULT_DEPLOYMENT_CONFIG

  log_proc_msg "Updating replicas"
  sed -i 's#{{REPLICAS}}#'$vault_replicas'#g' $VAULT_DEPLOYMENT_CONFIG

  log_proc_msg "Updating image name"
  sed -i 's#{{VAULT_IMAGE}}#'$VAULT_IMAGE'#g' $VAULT_DEPLOYMENT_CONFIG

  log_proc_msg "Updating port"
  sed -i 's#{{VAULT_PORT}}#'$VAULT_PORT'#g' $VAULT_DEPLOYMENT_CONFIG
}

__create_configmap() {
  log_proc_msg "Creating configmap for vault: $VAULT_CONFIGMAP"
  local vault_config=$(cat $VAULT_CONFIG)

  kubectl --kubeconfig $KUBECONFIG \
    create configmap "$VAULT_CONFIGMAP" \
    --from-literal=VAULT_LOCAL_CONFIG="$vault_config"
}

__cleanup_configmap() {
  log_proc_msg "Cleaning up configmap for vault: $VAULT_CONFIGMAP"

  kubectl --kubeconfig $KUBECONFIG \
    delete configmap \
    "$VAULT_CONFIGMAP" || true
}

__check_vault() {
  log_proc_msg "Checking vault container status on: $VAULT_IP:$VAULT_PORT"
  platform_exec "check_service_connection" "$VAULT_IP" "$VAULT_PORT" "$COMPONENT"
}

main() {
  __cleanup_configmap
  __create_configmap
  __update_deployment_config
  __cleanup_deployment
  __create_deployment
  __cleanup_service
  __create_service
  __update_vault_ip
  __check_vault
}

main
