#!/bin/bash
set -e
set -o pipefail

readonly RT_RESPONSE_FILE="$TMP_DIR/rt_response.tmp"

__rt_initialize() {
  if [ -z "$RT_USERNAME" ]; then
    log_proc_error "Missing artifactory username"
    exit 1
  fi

  if [ -z "$RT_PASSWORD" ]; then
    log_proc_error "Missing artifactory password"
    exit 1
  fi

  RT_API_URL="$RT_URL/api"
  RESPONSE_CODE=404
  RESPONSE_DATA=""
  CURL_EXIT_CODE=0

  touch $RT_RESPONSE_FILE
}

__rt_get() {
  __rt_initialize

  local url="$RT_API_URL/$1"
  local get_cmd="curl \
    -H \"Content-Type: application/json\" \
    -u $RT_USERNAME:$RT_PASSWORD \
    -X GET $url \
    --silent --write-out \"%{http_code}\n\" \
    --output $RT_RESPONSE_FILE"

  local proxy_args=""
  if [ ! -z "$RT_PROXY" ]; then
    proxy_args="--proxy $RT_PROXY"
  fi

  # auth is optional for proxy
  if [ ! -z "$proxy_args" ]; then
    if [ ! -z "$RT_PROXY_USERNAME" ] && [ ! -z "$RT_PROXY_PASSWORD" ]; then
      proxy_args="$RT_PROXY_USERNAME:$RT_PROXY_PASSWORD@$RT_PROXY"
    fi
  fi

  if [ ! -z "$proxy_args" ]; then
    get_cmd="$get_cmd $proxy_args"
  fi

  {
    RESPONSE_CODE=$(eval $get_cmd)
  } || {
    CURL_EXIT_CODE=$(echo $?)
  }

  is_request_success=false
  if [ $RESPONSE_CODE -ne 200 ]; then
    response="Failed request on $url. Error: $(cat $RT_RESPONSE_FILE)"
    response_status_code="$RESPONSE_CODE"
  elif [ $CURL_EXIT_CODE -gt 0 ]; then
    # we are assuming that if curl cmd failed, API is unavailable
    response="curl failed with error code $CURL_EXIT_CODE. Artifactory might be down."
    response_status_code=503
  else
    is_request_success=true
    response_status_code="$RESPONSE_CODE"
    response=$(cat $RT_RESPONSE_FILE)
  fi
}

__rt_put() {
  __rt_initialize
  local update="$2"

  local url="$RT_API_URL/$1"
  local put_cmd="curl \
    -H \"Content-Type: application/json\" \
    -u $RT_USERNAME:$RT_PASSWORD \
    -X PUT $url \
    -d '"$update"' \
    --silent --write-out \"%{http_code}\n\" \
    --output $RT_RESPONSE_FILE"

  local proxy_args=""
  if [ ! -z "$RT_PROXY" ]; then
    proxy_args="--proxy $RT_PROXY"
  fi

  # auth is optional for proxy
  if [ ! -z "$proxy_args" ]; then
    if [ ! -z "$RT_PROXY_USERNAME" ] && [ ! -z "$RT_PROXY_PASSWORD" ]; then
      proxy_args="$RT_PROXY_USERNAME:$RT_PROXY_PASSWORD@$RT_PROXY"
    fi
  fi

  if [ ! -z "$proxy_args" ]; then
    put_cmd="$put_cmd $proxy_args"
  fi

  {
    RESPONSE_CODE=$(eval $put_cmd)
  } || {
    CURL_EXIT_CODE=$(echo $?)
  }

  is_request_success=false
  if [ $RESPONSE_CODE -ne 200 ]; then
    response="Failed request on $url. Error: $(cat $RT_RESPONSE_FILE)"
    response_status_code="$RESPONSE_CODE"
  elif [ $CURL_EXIT_CODE -gt 0 ]; then
    # we are assuming that if curl cmd failed, API is unavailable
    response="curl failed with error code $CURL_EXIT_CODE. Artifactory might be down."
    response_status_code=503
  else
    is_request_success=true
    response_status_code="$RESPONSE_CODE"
    response=$(cat $RT_RESPONSE_FILE)
  fi
}

## GET calls ###################################
_rt_ping_server() {
  local ping_endpoint="system/ping"
  __rt_get "$ping_endpoint"
}

_rt_get_licenses() {
  local licenses_endpoint="system/licenses"
  __rt_get "$licenses_endpoint"
}

_rt_get_configs() {
  local config_endpoint="system/configuration"
  __rt_get "$config_endpoint"
}

_rt_get_proxies() {
  local proxy_endpoint="system/configuration/platform/proxies"
  __rt_get "$proxy_endpoint"
}

_rt_get_mailserver() {
  local mailserver_endpoint="system/configuration/platform/mail"
  __rt_get "$mailserver_endpoint"
}

_rt_get_repositories() {
  local repo_endpoint="repositories"
  __rt_get "$repo_endpoint"
}

## PUT calls ###################################

_rt_put_repository() {
  if [ "$#" -ne 2 ]; then
    log_proc_error "The number of arguments expected by put_repository is 2"
    log_proc_error "current arguments $#"
    exit 1
  fi

  local repository_name="$1"
  local body="$2"
  if [ -z "$repository_name" ]; then
    log_proc_error "Repository name is required"
    exit 1
  fi

  if [ -z "$body" ]; then
    log_proc_error "Repository configuration is required"
    exit 1
  fi

  local put_repo_endpoint="repositories/$repository_name"
  __rt_put "$put_repo_endpoint" "$body"

}
