#!/bin/bash
set -e
set -o pipefail

source $INSTALLER_LIB_DIR/installerAdapter.sh
source $INSTALLER_LIB_DIR/rtAdapter.sh

# Logger methods ##########################################
###########################################################
log_proc_marker() {
  local prompt="$@"
  echo ""
  echo "# $(date +"%T") #######################################"
  echo "# $prompt"
  echo "##################################################"
}

log_proc_msg() {
  local message="$@"
  echo "|___ $@"
}

log_proc_error() {
  local message="$1"
  local error="$2"
  local bold_red_text='\e[91m'
  local reset_text='\033[0m'

  echo -e "$bold_red_text|___ $message$reset_text"
  echo -e "     $error"
}

log_proc_warn() {
  local message="$@"
  local bold_yellow_text='\e[33m'
  local reset_text='\033[0m'

  echo -e "$bold_yellow_text|___ $message$reset_text"
}

log_proc_success() {
  local message="$@"
  local bold_green_text='\e[32m'
  local reset_text='\033[0m'

  echo -e "$bold_green_text|___ $message$reset_text"
}

print_help() {
  echo "

  Usage:
    ./pipelines <command> [flags]

  Examples:
    ./pipelines install

  Commands:
    install                 Run Pipelines installation
      --global-password       Set one password for all services (db, rabbitmq, installer). Can be changed later
      --install-user          User (and Group) that owns the installer generated files and folders ($USER by default) e.g. obie, obie:obie, 1001:obie, 1002:1002
      --installer-ip             External IP of the host
      --rt-url                Artifactory URL
      --rt-username           Artifactory admin username
      --rt-password           Artifactory admin password
      --rt-service-id         Service Id to register with Artifactory. Format 'jfrt@<id>'
      --rt-proxy              Proxy server to use for connecting to Artifactory
      --rt-proxy-username     User for the proxy server
      --rt-proxy-password     Password for proxy server
      --www-url               Use provided url for WWW instead of host IP
      --api-url               Use provided url for API instead of host IP
      --rabbitmq-url          URL to connect to rabbitmq with basic auth e.g. amqp://user:pass@myMsg.com
      --rabbitmq-admin-url    URL to connect to rabbitmq admin UI e.g. http://adminUser:adminPass@myMsg.com
      --image-registry-url    Image registry endpoint (default: docker.io/drydock)
      --image-registry-creds  Path to the json file containing registry credentials e.g. /home/ubuntu/.docker/config.json
      --state-bucket          Root bucket name for storing state
      --skip-image-pull       Do not pull images
      --no-verify-ssl         Pass -no-verify-ssl flag to services
    status                  Print status of all services
    restart                 Restart all services with updated configs
    upgrade                 Upgrade current installation
    info                    Print information about current installation on console
    clean                   Remove Pipelines components and files
    help                    Print this message
  "
  exit 0
}

# Helper methods ##########################################
###########################################################

declare -a SERVICE_IMAGES=("api" "www" "micro" "nexec")
declare -a CORE_IMAGES=("postgres" "vault" "rabbitmq" "redis")

bootstrap_installer() {
  log_proc_marker "Bootstrapping pipelines"

  ################## load env file  ###########################
  if [ ! -f "$PIPELINES_CONFIG" ]; then
    log_proc_msg "$PIPELINES_CONFIG does not exist, creating"
    mkdir -p $CONFIG_DIR
    cp -vr "$INSTALLER_SCRIPTS_DIR/configs/pipelines.env.template" "$PIPELINES_CONFIG"
    log_proc_msg "Successfully created pipelines config"
  else
    log_proc_msg "Loading $PIPELINES_CONFIG"
  fi

  ################## create logs directory ########################
  if [ ! -d "$LOGS_DIR" ]; then
    log_proc_msg "Creating logs directory"
    mkdir -p "$LOGS_DIR"
  fi

  ################## create temp directory  ########################
  if [ ! -d "$TMP_DIR" ]; then
    log_proc_msg "Creating tmp directory"
    mkdir -p "$TMP_DIR"
  fi

  ################## create master directory  ########################
  if [ ! -d "$CONFIG_DIR/master" ]; then
    log_proc_msg "Creating master directory"
    mkdir -p "$CONFIG_DIR/master"
  fi

  source "$PIPELINES_CONFIG"
}

bootstrap_platform_config() {
  log_proc_marker "Setting platform config"
  if [ -z "$OPERATING_SYSTEM" ] || [ -z "$ARCHITECTURE" ]; then
    source /etc/os-release
    sed -i '/^OPERATING_SYSTEM/d' "$PIPELINES_CONFIG"
    sed -i '/^ARCHITECTURE/d' "$PIPELINES_CONFIG"

    local architecture=$(uname -m)
    if [ "$NAME" == "Red Hat Enterprise Linux Server" ]; then
      # For RHEL, the OS value is defined as "Red Hat Enterprise Linux Server". We want
      # the want this to be consistent with the other OS names, so we will be calling
      # this RHEL. Also in RHEL version comes as major.minor but we want to have
      # only major version in our directory name so trimming to have only major version only.
      local os="RHEL"
      local os_version=$(echo "$VERSION_ID" | cut -f 1 -d '.')
    else
      # For CentOS, the OS value is defined as "CentOS Linux". We only want
      # the first part of it, so ignore everything after a space. Ubuntu defines
      # it as "Ubuntu" so that will not change here.
      local os=$(echo "$NAME" | cut -f 1 -d ' ')
      local os_version="$VERSION_ID"
    fi

    local operating_system=$os"_"$os_version

    ### default architecture & os to x86_64, Ubuntu_14.04, if empty
    if [ -z "$architecture" ]; then
      architecture="x86_64"
    fi
    if [ -z "$operating_system" ]; then
      operating_system="Ubuntu_16.04"
    fi

    echo "ARCHITECTURE=\"$architecture\"" >> "$PIPELINES_CONFIG"
    echo "OPERATING_SYSTEM=\"$operating_system\"" >> "$PIPELINES_CONFIG"
    log_proc_msg "ARCHITECTURE: $architecture"
    log_proc_msg "OS: $operating_system"
  fi

  if command -v "docker" &>/dev/null; then
    sed -i '/^DOCKER_VERSION/d' "$PIPELINES_CONFIG"

    local version=$(docker version --format '{{.Server.Version}}')
    # split and extract major.minor version eg. to extract 17.06 from 17.06.0-ce
    local docker_version="${version%.*}"

    echo "DOCKER_VERSION=\"$docker_version\"" >> "$PIPELINES_CONFIG"
    log_proc_msg "DOCKER_VERSION: $docker_version"
  fi

  source "$PIPELINES_CONFIG"
}

platform_exec() {
  local platform_call="$1"
  shift
  ## This is needed because the global IFS converts arguments into a list of
  ## strings, each in a new line
  local platform_call_args=$(echo "$@" | tr '\n' ' ')
  log_proc_msg "Executing $platform_call"

  local platform_script="$INSTALLER_SCRIPTS_DIR/$ARCHITECTURE/$OPERATING_SYSTEM/$platform_call.sh"
  local platform_helpers="$INSTALLER_SCRIPTS_DIR/$ARCHITECTURE/$OPERATING_SYSTEM/_helpers.sh"
  if [ -f "$platform_script" ]; then
    ## NOTE: we cannot parse arguments in "source .." command because of which
    ## the function is first source-ed and then eval-ed.
    source "$platform_script"
    eval "$platform_call" "$platform_call_args"
  elif [ -f "$platform_helpers" ]; then
    source "$platform_helpers"
    eval "$platform_call" "$platform_call_args"
  else
    log_proc_msg "Cannot execute $platform_call, missing file $platform_helpers"
    exit 1
  fi
}

set_config() {
  local config_key="$1"
  local config_value="$2"

  if [ -z "$config_key" ]; then
    log_proc_error "Cannot set empty key in config. Syntax 'set_config KEY VALUE'"
    exit 1
  fi

  if [ -z "$config_value" ]; then
    log_proc_error "Cannot set empty value in config for $config_key. Syntax 'set_config KEY VALUE'"
    exit 1
  fi

  sed -i 's#.*'"$config_key"'=.*#'"$config_key"'="'$config_value'"#g' $PIPELINES_CONFIG

  source "$PIPELINES_CONFIG"
}

check_installer_connection() {
  log_proc_msg "Checking if installer container is running"
  local wait_time=3
  _get_installer
  if [ $response_status_code -eq 503 ]; then
    log_proc_msg "Installer not running, retrying in $wait_time seconds"
    sleep $wait_time
    check_installer_connection
  else
    log_proc_success "Installer successfully running"
  fi
}

validate_ip() {
  local ip=$1
  local rx='([1-9]?[0-9]|1[0-9]{2}|2[0-4][0-9]|25[0-5])'
  [[ $ip =~ ^(http(s)?://)?([a-zA-Z0-9-]+(\.[a-zA-Z0-9-]+)+[a-zA-Z0-9/?]*)$ ||  $ip =~ ^$rx\.$rx\.$rx\.$rx$ ]];
}

validate_db_password() {
  local pwd=$1
  if [[ "$pwd" =~ [^-_+=a-zA-Z0-9] ]]; then
    return 1
  fi
  return 0
}

log_system_info() {
  local format="$1"
  if [ "$format" == "json" ]; then
    local jq_args=( )
    local jq_query='.'

    jq_args+=( --arg "release" "$RELEASE" )
    jq_query+=" | .release=\$release"

    jq_args+=( --arg "installMode" "$RUN_MODE" )
    jq_query+=" | .installMode=\$installMode"

    jq_args+=( --arg "artifactoryUrl" "$RT_URL" )
    jq_query+=" | .artifactoryUrl=\$artifactoryUrl"

    jq_args+=( --arg "artifactoryServiceId" "$RT_SERVICE_ID" )
    jq_query+=" | .artifactoryServiceId=\$artifactoryServiceId"

    jq_args+=( --arg "uiUrl" "$WWW_EXTERNAL_URL" )
    jq_query+=" | .uiUrl=\$uiUrl"

    jq_args+=( --arg "apiUrl" "$API_EXTERNAL_URL" )
    jq_query+=" | .apiUrl=\$apiUrl"

    jq_args+=( --arg "databaseEndpoint" "$DB_IP:$DB_PORT" )
    jq_query+=" | .databaseEndpoint=\$databaseEndpoint"

    jq_args+=( --arg "vaultServer" "$VAULT_IP" )
    jq_query+=" | .vaultServer=\$vaultServer"

    jq_args+=( --arg "rabbitMQServer" "$RABBITMQ_IP" )
    jq_query+=" | .rabbitMQServer=\$rabbitMQServer"

    jq_args+=( --arg "redisServer" "$REDIS_IP" )
    jq_query+=" | .redisServer=\$redisServer"

    jq_args+=( --arg "lastUpgrade" "$LAST_UPDATE_TIME" )
    jq_query+=" | .lastUpgrade=\$lastUpgrade"

    jq_args+=( --arg "operatingSystem" "$OPERATING_SYSTEM" )
    jq_query+=" | .operatingSystem=\$operatingSystem"

    jq_args+=( --arg "architecture" "$ARCHITECTURE" )
    jq_query+=" | .architecture=\$architecture"

    jq "${jq_args[@]}" "$jq_query" <<<'{}'
  else
    log_proc_marker "System information"
    log_proc_msg "Release: $RELEASE"
    log_proc_msg "Install mode: $RUN_MODE"
    log_proc_msg "Last upgrade: $LAST_UPDATE_TIME"
    log_proc_msg "Operating system: $OPERATING_SYSTEM"
    log_proc_msg "Architecture: $ARCHITECTURE"
    log_proc_msg "Artifactory URL: $RT_URL"
    log_proc_msg "Artifactory Service ID: $RT_SERVICE_ID"
    if [ ! -z "$WWW_EXTERNAL_URL" ]; then
      log_proc_msg "UI URL: $WWW_EXTERNAL_URL"
    fi
    if [ ! -z "$API_EXTERNAL_URL" ]; then
      log_proc_msg "API URL: $API_EXTERNAL_URL"
    fi
    log_proc_msg "Database Endpoint: $DB_IP:$DB_PORT"
    log_proc_msg "Vault Server: $VAULT_IP"
    log_proc_msg "RabbitMQ Server: $RABBITMQ_IP"
    log_proc_msg "Redis Server: $REDIS_IP"
  fi
}
