#!/bin/bash -e

# Helper methods ##########################################
###########################################################

export PLATFORM_DEPENDENCIES_DIR="$DEPENDENCIES_DIR/x86_64"

__install_python() {
  ################## Install python ######################################
  if type python &> /dev/null && true; then
    log_proc_msg "'python' already installed"
  else
    log_proc_msg "Installing 'python'"
    yum update -y
    yum install -y python
  fi
}

__install_jq() {
  ################## Install jq  #########################################
  if type jq &> /dev/null && true; then
    log_proc_msg "'jq' already installed"
  else
    log_proc_msg "Installing 'jq'"
    local jq_location="$PLATFORM_DEPENDENCIES_DIR/bin/jq"
    cp -vr "$jq_location" "/usr/bin/jq"
  fi
}

__install_yq() {
  ################## Install yq #######################################
  if type yq &> /dev/null && true; then
    log_proc_msg "'yq' already installed"
  else
    log_proc_msg "Installing 'yq'"
    local yq_location="$PLATFORM_DEPENDENCIES_DIR/bin/yq"
    cp -vr "$yq_location" "/usr/bin/yq"
  fi
}

__install_curl() {
  ################## Install curl  #######################################
  if type curl &> /dev/null && true; then
    log_proc_msg "'curl' already installed"
  else
    log_proc_msg "Installing 'curl'"
    local curl_rpm="$PLATFORM_DEPENDENCIES_DIR/yum/curl-7.29.0-51.el7.x86_64.rpm"
    rpm -i "$curl_rpm"
  fi
}

__install_nc() {
  ################## Install nc  #######################################
  if type nc &> /dev/null && true; then
    log_proc_msg "'nc' already installed"
  else
    log_proc_msg "Installing 'nc'"
    local nc_zip="$PLATFORM_DEPENDENCIES_DIR/yum/nmap-ncat-6.40-16.tar.gz"
    local nc_yum="$TMP_DIR/nc"
    mkdir -p "$nc_yum"
    tar -xvzf "$nc_zip" -C "$nc_yum"
    pushd $nc_yum
    rpm -i *.rpm
    popd
  fi
}

__install_psql() {
  if type psql &> /dev/null && true; then
    log_proc_msg "'psql' already installed"
  else
    log_proc_msg "Installing psql client"
    local psql_zip="$PLATFORM_DEPENDENCIES_DIR/yum/postgresql-9.2.24-1.tar.gz"
    local psql_yum="$TMP_DIR/psql"
    mkdir -p "$psql_yum"
    tar -xvzf "$psql_zip" -C "$psql_yum"
    pushd $psql_yum
    rpm -i *.rpm
    popd
  fi
}

__install_docker() {
  ################## Install Docker  #####################################
  if type docker &> /dev/null && true; then
    log_proc_msg "'docker' already installed"
  else
    log_proc_error "'docker' not installed. Please install docker to continue installation"
    exit 1
  fi
}

__install_docker_compose() {
  if [ "$RUN_MODE" == "devmode" ] || [ "$RUN_MODE" == "onebox" ]; then
    log_proc_msg "Installing docker compose"
    if type docker-compose &> /dev/null && true; then
      log_proc_msg "'docker-compose' already installed"
    else
      local compose_binary="$PLATFORM_DEPENDENCIES_DIR/bin/docker-compose"
      cp -vr $compose_binary /usr/bin/docker-compose
      docker-compose --version
    fi
  else
    log_proc_msg "Docker compose will not be installed in non-onebox mode"
  fi
}

install_dependencies() {
  log_proc_marker "Installing dependencies"
  if [ "$EUID" -ne 0 ]; then
    log_proc_warn "Skipping dependency installation since user doesn't have admin privileges"
  else
    __install_python
    __install_jq
    __install_yq
    __install_curl
    __install_nc
    __install_psql
    __install_docker
    __install_docker_compose
  fi
}

pull_service_images() {
  if [ "$SKIP_IMAGE_PULL" == true ]; then
    return
  fi

  log_proc_marker "Pulling service images"
  local image_tag="$RELEASE"
  if [ ! -z "$1" ]; then
    image_tag="$1"
  fi

  if [ "$RUN_MODE" == "devmode" ] || [ "$RUN_MODE" == "onebox" ]; then
    docker pull "$IMAGE_REGISTRY_URL/$DB_IMAGE:$image_tag"
  fi
  docker pull "$IMAGE_REGISTRY_URL/$VAULT_IMAGE:$image_tag"
  docker pull "$IMAGE_REGISTRY_URL/$MSG_IMAGE:$image_tag"
  docker pull "$IMAGE_REGISTRY_URL/$REDIS_IMAGE:$image_tag"
  docker pull "$IMAGE_REGISTRY_URL/$INSTALLER_IMAGE:$image_tag"
  docker pull "$IMAGE_REGISTRY_URL/$API_IMAGE:$image_tag"
  docker pull "$IMAGE_REGISTRY_URL/$WWW_IMAGE:$image_tag"
  docker pull "$IMAGE_REGISTRY_URL/$MICRO_IMAGE:$image_tag"

  log_proc_marker "All images pulled successfully"
}

generate_ssh_keypair() {
  local username=$(id -un $INSTALL_USER)
  local user_private_key="/home/$username/.ssh/id_rsa"
  local user_public_key="/home/$username/.ssh/id_rsa.pub"

  if [ -f "$SSH_PRIVATE_KEY" ] && [ -f $SSH_PUBLIC_KEY ]; then
    log_proc_msg "SSH keys already present, skipping"
  elif [ -f "$user_private_key" ] && [ -f "$user_public_key" ]; then
    log_proc_msg "User ssh keys present, copying them"
    cp -vr "$user_private_key" "$SSH_PRIVATE_KEY"
    cp -vr "$user_public_key" "$SSH_PUBLIC_KEY"
  else
    if type ssh-keygen &> /dev/null && true; then
      log_proc_msg "SSH keys not available, generating"
      local keygen_exec=$(ssh-keygen -t rsa -P "" -f $SSH_PRIVATE_KEY)
      log_proc_msg "SSH keys successfully generated"
    else
      log_proc_error "SSH keys cannot be generated. Missing 'ssh-keygen' command"
      log_proc_error "Either install 'ssh-keygen' or update keys manually"
      log_proc_error "Expecting private key at: $user_private_key"
      log_proc_error "Expecting public key at: $user_public_key"
      exit 1
    fi
  fi
}

# accepts arguments $host $port $serviceName $timeout
# $timeout is optional and defaults to 60 seconds if not defined
check_service_connection() {

  if [ "$#" != "3" ] && [ "$#" != "4" ]; then
    log_proc_error "invalid arguments to check_service_connection()"
    log_proc_error "Usage: check_service_connection <host> <port> <serviceName> <timeout (optional)>"
    exit 1
  fi

  local timeout=$4
  if [ -z "$timeout" ]; then
    timeout=60
  fi
  local host=$1
  if [ -z "$host" ]; then
    log_proc_error "got empty host in __check_service_connection(), exiting..."
    exit 1
  fi
  local port=$2
  if [ -z "$port" ]; then
    log_proc_error "got empty port in __check_service_connection(), exiting..."
    exit 1
  fi
  local service=$3
  if [ -z "$service" ]; then
    log_proc_error "got empty service name in __check_service_connection(), exiting..."
    exit 1
  fi
  local interval=3
  local counter=0
  local service_booted=false

  while [ $service_booted != true ] && [ $counter -lt $timeout ]; do
    if nc -vz $host $port &>/dev/null; then
      log_proc_msg "$service found"
      sleep 5
      service_booted=true
    else
      log_proc_msg "Waiting for $service to start"
      let "counter = $counter + $interval"
      sleep $interval
    fi
  done
  if [ $service_booted = false ]; then
    log_proc_error "Could not detect $service container for host:$host, port:$port"
    exit 1
  fi
}
